--[[
RealisticSeederUtil

Utility for my Realistic Seeder

Author:		Ifko[nator]
Datum:		15.06.2021
Version:	v1.4

History:	v1.0 @12.09.2020 - initial implementation in FS 19
			-------------------------------------------------------------------------------------
			v1.1 @15.02.2021 - reworkt changeMaterial function
			-------------------------------------------------------------------------------------
			v1.2 @06.06.2021 - added "getFertilizerFillUnitIndex" and "getSeedFillTypes" function
			-------------------------------------------------------------------------------------
			v1.3 @13.06.2021 - minior adjustments
			-------------------------------------------------------------------------------------
			v1.4 @15.06.2021 - added "getIsValidPalletName" function
]]

RealisticSeederUtil = {};
RealisticSeederUtil.currentModName = g_currentModName;
RealisticSeederUtil.languageShort = g_languageShort;
RealisticSeederUtil.currentModDirectory = g_currentModDirectory;

RealisticSeederUtil.validStoreCategories = {
    "seeders",
    "planters",
	"potatoHarvesting",
	"weeders",
	"discHarrows",
	"cultivators"
};

RealisticSeederUtil.modDesc = loadXMLFile("modDesc", RealisticSeederUtil.currentModDirectory .. "modDesc.xml");
RealisticSeederUtil.fillTypesPath = RealisticSeederUtil.currentModDirectory .. getXMLString(RealisticSeederUtil.modDesc, "modDesc.fillTypes#filename");
	
if fileExists(RealisticSeederUtil.fillTypesPath) then
	RealisticSeederUtil.fillTypesXML = loadXMLFile("map", RealisticSeederUtil.fillTypesPath);
end;

getfenv(0)["RealisticSeederUtil"] = RealisticSeederUtil;

function RealisticSeederUtil.getDebugPriority(xmlFile, path)
	return Utils.getNoNil(getXMLInt(xmlFile, path), 0);
end;

function RealisticSeederUtil.printError(errorMessage, isWarning, isInfo, luaName)
	local prefix = "::ERROR:: ";
	
	if isWarning then
		prefix = "::WARNING:: ";
	elseif isInfo then
		prefix = "::INFO:: ";
	end;
	
	print(prefix .. "from the " .. luaName .. ".lua: " .. tostring(errorMessage));
end;

function RealisticSeederUtil.printDebug(debugMessage, priority, addString, luaName)
	if priority >= 1 then
		local prefix = "";
		
		if addString then
			prefix = "::DEBUG:: from the " .. luaName .. ".lua: ";
		end;
		
		print(prefix .. tostring(debugMessage));
	end;
end;

function RealisticSeederUtil.getSpecByName(self, specName, currentModName)
    local spec = self["spec_" .. Utils.getNoNil(currentModName, RealisticSeederUtil.currentModName) .. "." .. specName];

	if spec ~= nil then
        return spec;
    end;

    return self["spec_" .. specName];
end;

function RealisticSeederUtil.getIsValidStoreCategory(storeCategory)
    for _, validStoreCategory in pairs(RealisticSeederUtil.validStoreCategories) do
        if storeCategory == validStoreCategory then
            return true;
        end;
    end;

    return false;
end;

function RealisticSeederUtil.getIsValidPalletName(palletName)
	return (not string.find(palletName, "PALLETFORK") and not string.find(palletName, "TRIGGER")) and (string.find(palletName, "PALLET") or string.find(palletName, "BIGBAG") or string.find(palletName, "TANK"));
end;

function RealisticSeederUtil.foundFunctionAdditionalTank(xmlFile)
    local foundFunctionAdditionalTank = false;
	local additionalTankNumber = 0;

	while true do
		local functionAditionalTankKey = "vehicle.storeData.functions.function(" .. additionalTankNumber .. ")";
		
		if not hasXMLProperty(xmlFile, functionAditionalTankKey) then
			break;
		end;

		if getXMLString(xmlFile, functionAditionalTankKey) == "$l10n_function_additionalTank" then
			foundFunctionAdditionalTank = true;

			break;
		end;

		additionalTankNumber = additionalTankNumber + 1;
    end;
    
    return foundFunctionAdditionalTank;
end;

function RealisticSeederUtil.getSeedFillUnitIndex(xmlFile, object, specFillUnit)
	local seedFillUnitIndex = 1;
	
	local baseKey = string.format("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(%d).fillUnits", Utils.getNoNil(object.configurations["fillUnit"], 1) - 1);

	local fillUnitNumber = 0;

	while true do
		local fillUnitKey = string.format("%s.fillUnit(%d)", baseKey, fillUnitNumber);
		
        if not hasXMLProperty(xmlFile, fillUnitKey) then
            break;
		end;

		for fillTypeIndex, _ in pairs(specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes) do
			if g_fillTypeManager:getFillTypeByIndex(fillTypeIndex).name == "SEEDS" then
				seedFillUnitIndex = fillUnitNumber + 1;

				break;
			end;
		end;

		fillUnitNumber = fillUnitNumber + 1;
    end;
    
    return seedFillUnitIndex;
end;

function RealisticSeederUtil.getFertilizerFillUnitIndex(xmlFile, object, specFillUnit)
	local fertiLizerFillUnitIndex = 1;
	
	local baseKey = string.format("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(%d).fillUnits", Utils.getNoNil(object.configurations["fillUnit"], 1) - 1);

	local fillUnitNumber = 0;

	while true do
		local fillUnitKey = string.format("%s.fillUnit(%d)", baseKey, fillUnitNumber);
		
        if not hasXMLProperty(xmlFile, fillUnitKey) then
            break;
		end;

		for fillTypeIndex, _ in pairs(specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes) do
			if string.find(g_fillTypeManager:getFillTypeByIndex(fillTypeIndex).name, "FERTILIZER") then
				fertiLizerFillUnitIndex = fillUnitNumber + 1;

				break;
			end;
		end;

		fillUnitNumber = fillUnitNumber + 1;
    end;
    
    return fertiLizerFillUnitIndex;
end;

function RealisticSeederUtil.changeMaterial(object, materialNode, translateNode)
	if object.components == nil then
		local newMaterial = getMaterial(translateNode, 0);
		local numChildren = getNumOfChildren(materialNode);

		if numChildren > 0 then
			for childNumber = 0, numChildren - 1 do
				local oldMaterial = getMaterial(getChildAt(materialNode, childNumber), 0);

				ConfigurationUtil.replaceMaterialRec(object, getChildAt(materialNode, childNumber), oldMaterial, newMaterial);
			end;
		else
			local oldMaterial = getMaterial(materialNode, 0);

			ConfigurationUtil.replaceMaterialRec(object, materialNode, oldMaterial, newMaterial);
		end;
	else
		local oldMaterial = getMaterial(materialNode, 0);
		local newMaterial = getMaterial(translateNode, 0);
	
		for _, component in pairs(object.components) do
			ConfigurationUtil.replaceMaterialRec(object, component.node, oldMaterial, newMaterial);
		end;
	end;
end;

function RealisticSeederUtil.getCanFruitBePlanted(fillType, fruitType)
	local canFruitBePlanted = false;

    if g_currentMission.fruits ~= nil then
        if fruitType ~= nil then
			if fruitType ~= nil 
				and g_currentMission.fruits[fruitType] ~= nil
				and g_currentMission.fruits[fruitType].id ~= nil 
				and g_currentMission.fruits[fruitType].id ~= 0 
			then
				canFruitBePlanted = true;
			end;
		else
			if fillType ~= nil then
				local seedsFruitType = g_fruitTypeManager:getFruitTypeByName(string.sub(g_fillTypeManager:getFillTypeByIndex(fillType).name, 6));
				
				if seedsFruitType ~= nil 
					and g_currentMission.fruits[seedsFruitType.index] ~= nil 
					and g_currentMission.fruits[seedsFruitType.index].id ~= nil 
					and g_currentMission.fruits[seedsFruitType.index].id ~= 0 
				then
					canFruitBePlanted = true;
				end;
			end;
		end;
		
		
	end;

	return canFruitBePlanted;
end;

function RealisticSeederUtil.getL10nText(text)
	if text:sub(1, 6) == "$l10n_" then
		text = g_i18n:getText(text:sub(7));
	elseif g_i18n:hasText(text) then
		text = g_i18n:getText(text);
	end;

	return text;
end;

function RealisticSeederUtil.getSeedFillTypes()
	local fillTypesXML = loadXMLFile("map", RealisticSeederUtil.currentModDirectory .. getXMLString(RealisticSeederUtil.modDesc, "modDesc.fillTypes#filename"));

	local fillTypes = nil;
	local fillTypeNames = "";
    local fillTypeNumber = 0;

    while true do
		local fillTypeKey = "map.fillTypes.fillType(" .. tostring(fillTypeNumber) .. ")";

		if not hasXMLProperty(fillTypesXML, fillTypeKey) then
			break;
		end;

        local fillType = Utils.getNoNil(getXMLString(fillTypesXML, fillTypeKey .. "#name"), "");

        if fillType ~= "" then
			if fillTypeNames ~= "" then 
				fillType = " " .. fillType;
			end;

			fillTypeNames = fillTypeNames .. fillType;
		end;
        
        fillTypeNumber = fillTypeNumber + 1;
    end;

	fillTypes = g_fillTypeManager:getFillTypesByNames(fillTypeNames, "Warning: invalid fillType '%s'.");

	return fillTypes;
end;